#pragma once

#include <unordered_map>
#include <vector>
#include <string>
#include <fstream>

std::unordered_map<std::string, float> amino_acid_radii = {
    {"ARG", 6.6}, {"HIS", 6.1}, {"LYS", 6.4}, {"ASP", 5.6}, {"GLU", 5.9}, {"SER", 5.2}, {"THR", 5.6},
    {"ASN", 5.7}, {"GLN", 6.0}, {"CYS", 5.5}, {"GLY", 4.5}, {"PRO", 5.6}, {"ALA", 5.0}, {"VAL", 5.9},
    {"ILE", 6.2}, {"LEU", 6.2}, {"MET", 6.2}, {"PHE", 6.4}, {"TYR", 6.5}, {"TRP", 6.8}
};

std::unordered_map<std::string, float> amino_acid_masses = {
    {"ARG", 156.19}, {"HIS", 137.14}, {"LYS", 128.18}, {"ASP", 115.09}, {"GLU", 129.12}, {"SER",  87.08},
    {"THR", 101.11}, {"ASN", 114.11}, {"GLN", 128.13}, {"CYS", 103.15}, {"GLY",  57.05}, {"PRO",  97.12},
    {"ALA",  71.08}, {"VAL",  99.13}, {"ILE", 113.16}, {"LEU", 113.16}, {"MET", 131.20}, {"PHE", 147.18},
    {"TYR", 163.18}, {"TRP", 186.22}
};

std::unordered_map<std::string, float> amino_acid_charges = {
    {"ARG", 1.0}, {"HIS", 0.5}, {"LYS", 1.0}, {"ASP",-1.0}, {"GLU",-1.0}, {"SER", 0.0}, {"THR", 0.0},
    {"ASN", 0.0}, {"GLN", 0.0}, {"CYS", 0.0}, {"GLY", 0.0}, {"PRO", 0.0}, {"ALA", 0.0}, {"VAL", 0.0},
    {"ILE", 0.0}, {"LEU", 0.0}, {"MET", 0.0}, {"PHE", 0.0}, {"TYR", 0.0}, {"TRP", 0.0}
};

struct cg_pdb {
	std::vector<float> m_x, m_y, m_z, m_r, m_mass, m_charge;
    std::vector<std::string> m_amino_acid;
    float m_total_mass;
	bool is_failed;

	cg_pdb(const std::string& filename) : is_failed(false) {
		std::ifstream handle(filename.c_str());
		if(!handle.is_open()) {
			is_failed = true;
			return;
		}
        // [ Read data from file ]
      	std::string line;
      	while(std::getline(handle, line)) {
			if(line.empty() || line.size() < 4) {
       			continue;
			}
          	if(line.substr(0, 4) == "ATOM" && line.substr(13, 2) == "CA") {
           		m_x.push_back(0.1 * std::stod(line.substr(30, 8)));
          		m_y.push_back(0.1 * std::stod(line.substr(38, 8)));
              	m_z.push_back(0.1 * std::stod(line.substr(46, 8)));
             	std::string type = line.substr(17, 3);
                m_amino_acid.push_back(type);
              	m_r.push_back(0.1 * amino_acid_radii[type]);
             	m_charge.push_back(amino_acid_charges[type]);
              	m_mass.push_back(amino_acid_masses[type]);
            }
		}
    	handle.close();
        // [ Move to center of mass ]
        float c_x = 0.0, c_y = 0.0, c_z = 0.0;
        for(int i=0; i<(int)m_x.size(); ++i) {
            c_x += m_x[i] * m_mass[i];
            c_y += m_y[i] * m_mass[i];
            c_z += m_z[i] * m_mass[i];
            m_total_mass += m_mass[i];
        }
        c_x /= m_total_mass;
        c_y /= m_total_mass;
        c_z /= m_total_mass;
        for(int i=0; i<(int)m_x.size(); ++i) {
            m_x[i] -= c_x; 
            m_y[i] -= c_y; 
            m_z[i] -= c_z; 
        }
	}
};
