#include "cg_pdb.h"

#include <cmath>
#include <iostream>
#include <algorithm>

inline bool is_within(const float x1, const float y1, const float z1, const float x2, const float y2, const float z2, const float r) {
    if((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) + (z2-z1)*(z2-z1) < r*r) {
        return true;
    }
    return false;
}

struct Volume : public cg_pdb {
    float m_pdb_volume;

    Volume(const std::string& filename, int N = 10000) : cg_pdb(filename), m_pdb_volume(0.0) {
        const float x_min = *std::min_element(m_x.begin(), m_x.end()); 
        const float x_max = *std::max_element(m_x.begin(), m_x.end()); 
        const float y_min = *std::min_element(m_y.begin(), m_y.end()); 
        const float y_max = *std::max_element(m_y.begin(), m_y.end()); 
        const float z_min = *std::min_element(m_z.begin(), m_z.end()); 
        const float z_max = *std::max_element(m_z.begin(), m_z.end());

        const float box_volume = (x_max - x_min) * (y_max - y_min) * (z_max - z_min); 
        const float dV = box_volume / N;
        const float dl = std::pow(dV, 1.0 / 3.0);

        int n = 0;

        /*std::cout << x_min << ", " << x_max << std::endl;
        std::cout << y_min << ", " << y_max << std::endl;
        std::cout << z_min << ", " << z_max << std::endl;
        std::cout << box_volume << ", " << dV << ", " << dl << std::endl;*/

        for(float x = x_min + dl / 2.0; x < x_max; x += dl) {
            for(float y = y_min + dl / 2.0; y < y_max; y += dl) {
                for(float z = z_min + dl / 2.0; z < z_max; z += dl) {
                    for(int i=0; i<(int)m_x.size(); ++i) {
                        if(is_within(x, y, z, m_x[i], m_y[i], m_z[i], m_r[i])) {
                            ++n;
                            break;
                        }
                    }
                }
            }
        }
        
        m_pdb_volume = (n / (float) N) * box_volume; 
    }
};


int main(int argc, const char* argv[]) {
    if(argc < 2) {
        std::cout << "Usage: ./volume <file.pdb> [file.pdb] ..." << std::endl;
        std::cout << std::endl;
        std::cout << "  They output will be a list of volumes in units of nm^3" << std::endl;
        return -1;
    }
    for(int i=1; i<argc; ++i) {
        Volume volume(argv[i]);
        std::cout << volume.m_pdb_volume << std::endl;
    }
    
    return 0;
}
